# coding: utf-8
'''
Created on 10.02.2015

@author: Kuzmin M.
'''

import json

from common.hierarchy import hasChildren, changeNodePositionInLevelOfHierarchy, \
    leftShiftNodeInHierarchy
from common.sysfunctions import toHexForXml, getGridHeight
from nci._nci_orm import control_form_typeCursor, education_formCursor
from ru.curs.celesta.showcase.utils import XMLJSONConverter
from umk._umk_orm import edu_contentCursor, umk_versionCursor, \
    complex_programCursor


try:
    from ru.curs.showcase.core.jython import JythonDTO
except:
    from ru.curs.celesta.showcase import JythonDTO

def getTree(context, main=None, add=None, filterinfo=None, session=None, elementId=None, sortColumnList=None, parentId=None):
    u'''Функция получения данных для tree-грида. '''

    # Получение курсора на таблицу
    currentTable = edu_contentCursor(context)
    currentTable2 = edu_contentCursor(context)
    control_form_type = control_form_typeCursor(context)
    education_form = education_formCursor(context)

    if "umkVersionId" in context.getData():
        umk_version = context.getData()["umkVersionId"]
    else:
        umk_version = None

    umk_ver = umk_versionCursor(context)
    umk_ver.get(umk_version)

    # Заголовки полей
    _headers = {'id': '~~id',
                'dewey':u'Номер',
                'name': u'Название элемента',
                'type':u'Тип',
                'hours':u'Кол-во часов',
                'hours_diff':u'Разница в часах между данным уровнем и суммой часов потомков',
                'control_form': u'Форма контроля',
                'education_form': u'Рекомендованная форма обучения',
                'HasChildren': 'HasChildren',
                'properties': 'properties'}
    # Типы элементов из CelestaDoc
    table_meta = currentTable.meta()
    column_name = 'type'
    typeDict = json.loads(table_meta.getColumn(column_name).getCelestaDoc())

    for col in _headers:
        _headers[col] = toHexForXml(_headers[col])
    currentTable.setRange("umk_version", umk_version)
    currentTable2.setRange("umk_version", umk_version)
    totalcount = currentTable.count()
    if totalcount != 0:
        data = {"records": {"rec": []}}

        # Событие по одинарному клику по записи
        event = {"event":
                    {"@name":"row_single_click",
                     "action": {
                        '#sorted': [
                            {"main_context": "current"},
                            {"datapanel":
                                {'@type':"current",
                                 '@tab':"current"
                                 }}]
                        }
                     }
                 }
        # Сортировка по умолчанию
        if not parentId:
            # Обработка иерархического справочника
            currentTable.setFilter('dewey', "!%'.'%")
            currentTable.orderBy('sortvalue asc')
            for rec in currentTable.iterate():
                row = {}
                len_dewey = len(rec.dewey.split('.'))
                if len_dewey == 1:
                    row[_headers["id"]] = rec.uid
                    row[_headers["dewey"]] = '%s.' % rec.dewey
                    row[_headers["name"]] = {'b': rec.name}
                    if rec.type:
                        row[_headers["type"]] = {'b': typeDict['%s' % rec.type]}
                    row[_headers["hours"]] = {'b': rec.hours}
                    # сумма часов более нижнего уровня
                    row[_headers["hours_diff"]] = 0
                    currentTable2.setFilter('dewey', "('%s.'%%)&(!'%s.'%%'.'%%)" % (rec.dewey, rec.dewey))
                    for rec2 in currentTable2.iterate():
                        row[_headers["hours_diff"]] += float(rec2.hours) if rec2.hours else 0
                    row[_headers["hours_diff"]] = float(rec.hours) - row[_headers["hours_diff"]]
                    if control_form_type.tryGet(rec.control_form):
                        row[_headers["control_form"]] = {'b': control_form_type.name}
                    else:
                        row[_headers["control_form"]] = ''
                    if education_form.tryGet(rec.education_form):
                        row[_headers["education_form"]] = {'b': education_form.name}
                    else:
                        row[_headers["education_form"]] = ''
                    row[_headers["HasChildren"]] = '1' if hasChildren(context, rec, "dewey") else '0'
                    row[_headers["properties"]] = event
                    data["records"]["rec"].append(row)
        else:
            currentTable.get(parentId)
            parent = currentTable.dewey
            currentTable.setFilter('dewey', "('%s.'%%)&(!'%s.'%%'.'%%)" % (parent, parent))
            currentTable.orderBy('sortvalue asc')
            for rec in currentTable.iterate():
                row = {}
                row[_headers["id"]] = rec.uid
                row[_headers["dewey"]] = '%s.' % rec.dewey
                row[_headers["name"]] = rec.name
                if rec.type:
                    row[_headers["type"]] = typeDict['%s' % rec.type]
                row[_headers["hours"]] = rec.hours
                # сумма часов более нижнего уровня
                row[_headers["hours_diff"]] = 0
                currentTable2.setFilter('dewey', "('%s.'%%)&(!'%s.'%%'.'%%)" % (rec.dewey, rec.dewey))
                for rec2 in currentTable2.iterate():
                    row[_headers["hours_diff"]] += float(rec2.hours) if rec2.hours else 0
                row[_headers["hours_diff"]] = float(rec.hours) - row[_headers["hours_diff"]]
                if control_form_type.tryGet(rec.control_form):
                    row[_headers["control_form"]] = control_form_type.name
                else:
                    row[_headers["control_form"]] = ''
                if education_form.tryGet(rec.education_form):
                    row[_headers["education_form"]] = education_form.name
                else:
                    row[_headers["education_form"]] = ''
                row[_headers["HasChildren"]] = '1' if hasChildren(context, rec, "dewey") else '0'
                if rec.type == 5:
                    event = {"event":
                                {"@name":"row_single_click",
                                    "action": {
                                      '#sorted': [
                                          {"main_context": "current"},
                                          {"datapanel":
                                            {'@type':"current",
                                             '@tab':"current",
                                             'element': {'@id': 'courseModuleGrid',
                                                         'add_context': ''
                                                         }
                                             }}]
                                    }
                                 }
                             }
                else:
                    event = {"event":
                                {"@name":"row_single_click",
                                 "action": {
                                    '#sorted': [
                                        {"main_context": "current"},
                                        {"datapanel":
                                            {'@type':"current",
                                             '@tab':"current",
                                             # 'element': {'@id': 'UCgeneralEorGrid',
                                             #            'add_context': 'hide'
                                             #            }
                                             }}]
                                    }
                                 }
                             }
                row[_headers["properties"]] = event
                data["records"]["rec"].append(row)
    else:
        data = {"records":""}
    # raise Exception(data)

    resultData = XMLJSONConverter.jsonToXml(json.dumps(data))

    # Формирование заголовка
    complex_program = complex_programCursor(context)
    complex_program.get(umk_ver.complex_program_id)
    dpwidth = json.loads(session)['sessioncontext']['currentDatapanelWidth']
    gridWidth = (int(dpwidth) - 50) if dpwidth not in ('0', '', 0) else "800"
    headerMess = u'<span style="width:%spx; font-size: 11pt; display:block; text-align:center; color: black; margin-top:-15px; margin-bottom:-10px;">Режим работы с УМК %s (версия %d)</span>' % (gridWidth, complex_program.name, umk_ver.number)

    meta = {"gridsettings":
                    {"labels":
                        {"header":
                            {"h3":headerMess}
                        },
                     "action":
                        {"#sorted":
                            [{"main_context":'current'},
                             {"datapanel":
                                {"@type":'current',
                                 "@tab":'current'
                                 }
                              }]
                         },
                     "columns":
                        {"col":
                            [{"@id":u'Номер', "@width":'65px'},
                             {"@id":u'Название элемента', "@width":'230px'},
                             {"@id":u'Тип', "@width":'45px'},
                             {"@id":u'Кол-во часов', "@width":'35px'},
                             {"@id":u'Разница в часах между данным уровнем и суммой часов потомков', "@width":'70px'},
                             {"@id":u'Форма контроля', "@width":'55px'},
                             {"@id":u'Рекомендованная форма обучения', "@width":'110px'},
                        ]
                         },
                     "properties":
                        {"@flip":"false",
                        "@pagesize":20,
                        "@gridWidth": "100%",    # "%spx" % gridWidth,
                        "@gridHeight":getGridHeight(session, numberOfGrids=2),
                        "@rowHeight":19,
                        "@profile":"default.properties",
                        }
                    }
                }
    settings = XMLJSONConverter.jsonToXml(json.dumps(meta))

    return JythonDTO(resultData, settings)

def gridToolBar(context, main=None, add=None, filterinfo=None, session=None, elementId=None):

    sessionjson = json.loads(session)["sessioncontext"]
    edu_content = edu_contentCursor(context)
    currentRecordId = ''
    type_name_edit, type_name_add = '', ''
    addstyle = "false"
    upStyle = "true"
    downStyle = "true"
    editstyle = "true"
    leftStyle = "true"
    if 'currentRecordId' in sessionjson['related']['gridContext']:
        editstyle = "false"
        upStyle = "false"
        downStyle = "false"
        currentId = sessionjson['related']['gridContext']['currentRecordId']
        # dpp_eus.get(currentId)
        # if dpp_eus.type == 4:
        #    leftStyle = 'false'
    includeStyle = 'true'
    '''
    if 'selectedRecordId' in sessionjson['related']['gridContext']:
        selectedRecs = sessionjson['related']['gridContext']['selectedRecordId']
        if not isinstance(selectedRecs,list):
            selectedRecs = [selectedRecs]
        moduleCount = 0
        styleFlag = True
        moduleDewey = None
        themeList = []
        for rec in selectedRecs:
            dpp_eus.get(rec)
            #Если раздел или элемент темы, то нельзя включать в занятие
            if dpp_eus.type == 1 or dpp_eus.type == 4:
                styleFlag = False
            elif dpp_eus.type == 2:
                moduleCount += 1
                if moduleCount > 1:
                    styleFlag = False
                moduleDewey = dpp_eus.dewey
            elif dpp_eus.type == 3:
                themeList.append(dpp_eus.dewey)
        
        #Проверям, что выбранные темы принадлежат одному модулю
        if moduleDewey is None:
            if themeList != []:
                moduleDewey = '.'.join(themeList[0].split('.')[:-1])
        if themeList != []:
            for theme in themeList:
                if '.'.join(theme.split('.')[:-1]) != moduleDewey:
                    styleFlag = False
                
        if styleFlag:
            includeStyle = 'false'
    '''

    data = {"gridtoolbar":
                {"#sorted":
                    [{"item":
                        {"@text":u"Добавить",
    #                    "@img":"bad.png",
                         "@hint":u"Добавить %s" % type_name_add,
                         "@disable":addstyle,
                         "action":
                             {"@show_in":"MODAL_WINDOW",
                              "#sorted":
                                [{"main_context":'current'},
                                 {"modalwindow":
                                    {"@caption":u"Добавление",
                                     "@height":"400",
                                     "@width":"600"
                                     }
                                  },
                                 {"datapanel":
                                    {"@type":"current",
                                     "@tab":"current",
                                     "element":
                                         {"@id":"eduContentCard",
                                          "add_context": "add"}
                                    }
                                 }]
                               }
                            }
                         },
                     {"item":
                        {"@text":u"Редактировать",
    #                    "@img":"bad.png",
                         "@hint":u"Редактировать %s" % type_name_edit,
                         "@disable":editstyle,
                         "action":
                             {"@show_in":"MODAL_WINDOW",
                              "#sorted":
                                [{"main_context":'current'},
                                 {"modalwindow":
                                    {"@caption":u"Редактирование",
                                     "@height":"400",
                                     "@width":"600"
                                     }
                                  },
                                 {"datapanel":
                                    {"@type":"current",
                                     "@tab":"current",
                                     "element":
                                         {"@id":"eduContentCard",
                                          "add_context": "edit"}
                                    }
                                 }]
                               }
                            }
                         },
                        {"item":
                        {"@text":u"Удалить",
    #                    "@img":"bad.png",
                         "@hint":u"Удалить выбранный элемент",
                         "@disable":editstyle,
                         "action":
                             {"@show_in":"MODAL_WINDOW",
                              "#sorted":
                                [{"main_context":'current'},
                                 {"modalwindow":
                                    {"@caption":u"Удаление",
                                     "@height":"150",
                                     "@width":"300"
                                     }
                                  },
                                 {"datapanel":
                                    {"@type":"current",
                                     "@tab":"current",
                                     "element":
                                         {"@id":"deleteFromEduContentGrid",
                                          "add_context": elementId}
                                    }
                                 }]
                               }
                            }
                         },
                         {"item":
                        {"@text":u"Массовый ввод элементов",
    #                    "@img":"bad.png",
                         "@hint":u"Данные из xslx",
                         "@disable":addstyle,
                         "action":
                             {"@show_in":"MODAL_WINDOW",
                              "#sorted":
                                [{"main_context":'current'},
                                 {"modalwindow":
                                    {"@caption":u"Массовый ввод элементов из xslx",
                                     "@height":"200",
                                     "@width":"450"
                                     }
                                  },
                                 {"datapanel":
                                    {"@type":"current",
                                     "@tab":"current",
                                     "element":
                                         {"@id":"eduContentUploadCard",
                                          "add_context": elementId}
                                    }
                                 }]
                               }
                            }
                         }

                       ]
                     }
                 }

    return XMLJSONConverter.jsonToXml(json.dumps(data))


def changeNumber(context, main=None, add=None, filterinfo=None,
             session=None, elementId=None, sortColumnList=None, firstrecord=None, pagesize=None):
    session = json.loads(session)["sessioncontext"]
    currentRec = session['related']['gridContext']['currentRecordId']
    umk_version = context.getData()["umkVersionId"]
    edu_content = edu_contentCursor(context)
    edu_content.get(currentRec)
    edu_content.setRange('umk_version', umk_version)
    # raise Exception(currentRecordId, int(main))

    changeNodePositionInLevelOfHierarchy(context, edu_content, 'dewey', 'sortvalue', 1 if add == 'down' else -1)


def changeLevel(context, main=None, add=None, filterinfo=None,
             session=None, elementId=None, sortColumnList=None, firstrecord=None, pagesize=None):
    session = json.loads(session)["sessioncontext"]
    currentRec = session['related']['gridContext']['currentRecordId']
    umk_version = context.getData()["umkVersionId"]
    edu_content = edu_contentCursor(context)
    edu_content.get(currentRec)
    edu_content.setRange('umk_version', umk_version)

    leftShiftNodeInHierarchy(context, edu_content, 'dewey', 'sortvalue')